/*
** Module   : LargeObjectOps.java
** Abstract : helper routines to process LOBs
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 GES 20130516 First version implementing an early version of the COPY-LOB statement which
**                  is suitable for executing testcases but is not yet 100% complete.
** 002 EVK 20140206 Added method readFromFile. This is an early version of COPY-LOB statement which
**                  is suitable for executing testcases but is not yet 100% complete.
** 003 VIG 20140914 Added comment about behaviour in readFromFile method.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * Provides static utility methods for processing large objects (LOBs) in a Progress 4GL
 * compatible manner.
 * <p> 
 * At this time, the following Progress 4GL features are handled:
 * <p>
 * <pre>
 * Progress                       Java Equivalent
 * ---------------------------    ---------------------------------------------------------
 * COPY-LOB                       writeToFile
 * </pre>
 */
public class LargeObjectOps
{
   /**
    * Private constructor to ensure instances are not created. All functionality of this class
    * is to be used via static methods.
    */
   private LargeObjectOps()
   {
   }

   /**
    * Read the full contents of the file to the given binary object.
    *
    * @param    fname
    *           The target filename.
    * @param    data
    *           The data to write.
    */
   public static void readFromFile(String fname, BinaryData data)
   {
// TODO: this is just a first pass, the real/full behavior should be implemented as following:
//      1. size(data) < size(file). For exapmle size(data) = 5 and size(file) = 23.
//      Progress reads all file and dynamically increases data size to needed value 
//      i.e. from 5 to 23 (in case of data).
//      Java impl only reads size(data) bytes from a file, i.e. 5
//      2. size(data) > size(file). For exapmle size(data) = 35 and size(file) = 23.
//      Both sides works equal except the one border case when the size of a file is 0.
//      In that case Progress works correctly without errors but Java implementation 
//      raises EndConditionException.
      
      if (data == null || data.isUnknown() || fname == null)
      {
         return;
      }

      Stream file = StreamFactory.openFileStream(fname, false, false);

      try
      {
         file.readBlock((memptr) data);
      }
      catch (EndConditionException | ErrorConditionException iioe)
      {
         throw new StopConditionException(iioe);
      }
      finally
      {
         file.close();
      }
   }

   /**
    * Read the full contents of the file to the given binary object.
    *
    * @param    fname
    *           The target filename.
    * @param    data
    *           The data to write.
    */
   public static void readFromFile(character fname, BinaryData data)
   {
      if (fname == null || fname.isUnknown())
      {
         return;
      }
      readFromFile(fname.toStringMessage(), data);
   }

   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    */
   public static void writeToFile(BinaryData data, character fname)
   {
      writeToFile(data, fname, false);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    * @param    append
    *           <code>true</code> if the contents should be appended to the file.
    */
   public static void writeToFile(BinaryData data, character fname, boolean append)
   {
      if (fname == null || fname.isUnknown())
      {
         return;
      }
      
      writeToFile(data, fname.toStringMessage(), append);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    */
   public static void writeToFile(BinaryData data, String fname)
   {
      writeToFile(data, fname, false);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    * @param    append
    *           <code>true</code> if the contents should be appended to the file.
    */
   public static void writeToFile(BinaryData data, String fname, boolean append)
   {
      // TODO: this is just a first pass, the real/full behavior needs to be determined
      
      if (data == null || data.isUnknown() || fname == null || fname.length() == 0)
      {
         return;
      }
      
      Stream file     = StreamFactory.openFileStream(fname, true, append);
      byte[] contents = data.getByteArray();
      
      try
      {
         // empty array should yield an empty file
         if (contents.length > 0)
         {
            file.write(contents);
         }
      }
      
      catch (InterruptedIOException iioe)
      {
         throw new StopConditionException(iioe);
      }
      
      catch (IOException ioe)
      {
         ErrorManager.showErrorAndAbend(140, Stream.PIPE_BROKEN_MESSAGE);
      }
      
      finally
      {
         file.close();
      }
   }
}
